package com.vctgo.system.service.impl;

import java.util.*;
import java.util.stream.Collectors;

import com.alibaba.nacos.common.utils.UuidUtils;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.vctgo.common.core.text.Convert;
import com.vctgo.common.core.utils.ServletUtils;
import com.vctgo.common.core.utils.StrUtils;
import com.vctgo.common.mybatisplus.aop.TemporaryIgnoreTables;
import com.vctgo.common.mybatisplus.constant.MybatisPageConstants;
import com.vctgo.system.domain.SysRoleDept;
import com.vctgo.system.domain.SysRoleMenu;
import com.vctgo.system.domain.SysUserRole;
import com.vctgo.system.mapper.*;
import com.vctgo.system.service.ISysRoleService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.vctgo.common.core.constant.UserConstants;
import com.vctgo.common.core.exception.ServiceException;
import com.vctgo.common.core.utils.SpringUtils;
import com.vctgo.common.datascope.annotation.DataScope;
import com.vctgo.common.security.utils.SecurityUtils;
import com.vctgo.system.api.domain.SysRole;
import com.vctgo.system.api.domain.SysUser;

/**
 * 角色 业务层处理
 *
 * @author vctgo
 */
@Service
public class SysRoleServiceImpl implements ISysRoleService {

    @Autowired
    private SysRoleMapper roleMapper;

    @Autowired
    private SysRoleMenuMapper roleMenuMapper;

    @Autowired
    private SysUserRoleMapper userRoleMapper;

    @Autowired
    private SysRoleDeptMapper roleDeptMapper;

    @Autowired
    private SysUserMapper sysUserMapper;

    /**
     * 根据条件分页查询角色数据
     *
     * @param role 角色信息
     * @return 角色数据集合信息
     */
    @TemporaryIgnoreTables
    @Override
    @DataScope(deptAlias = "d")
    public IPage<SysRole> selectRolePage(SysRole role) {
        Page mpPage = new Page(Convert.toLong(ServletUtils.getParameterToInt(MybatisPageConstants.PAGE_NUM), 1L)
                , Convert.toLong(ServletUtils.getParameterToInt(MybatisPageConstants.PAGE_SIZE), 10L));
        if (1 == SecurityUtils.getTenantType()) {
            SecurityUtils.setTemporaryIgnoreTables(Arrays.asList("d", "r", "ur", "u", "uu", "sur", "st", "sys_role",
                    "sys_user_role", "sys_user", "sys_dept"));
        }
        role.setAdminRole(false);
        role.setTimeOrder("1");
        role.setIsPage("1");
        role.setTenantType(SecurityUtils.getTenantType());
        IPage<SysRole> sysRoleIPage = roleMapper.selectRoleList(mpPage, role);
        if (!"3".equals(SecurityUtils.getTenantType()) && !sysRoleIPage.getRecords().isEmpty()) {
            SecurityUtils.setTemporaryIgnoreTables(Collections.singletonList("sys_user"));
            LambdaQueryWrapper<SysUser> sysUserWrapper = new LambdaQueryWrapper<>();
            sysUserWrapper.eq(SysUser::getDelFlag,"0");
            sysUserWrapper.in(SysUser::getUserId,sysRoleIPage.getRecords().stream().map(SysRole::getCreateById).collect(Collectors.toList()));
            List<SysUser> sysUserList = sysUserMapper.selectList(sysUserWrapper);
            sysRoleIPage.getRecords().forEach(sysRole -> {
                Optional<SysUser> sysUserOptional = sysUserList.stream().filter(sysUser -> sysUser.getUserId().longValue() == sysRole.getCreateById().longValue()).findFirst();
                sysUserOptional.ifPresent(sysUser -> sysRole.setCreateUserName(sysUser.getNickName()));
            });
            SecurityUtils.removeTemporaryIgnoreTables(Collections.singletonList("sys_user"));
        }
        return sysRoleIPage;
    }

    /**
     * 根据条件分页查询角色数据
     *
     * @param role 角色信息
     * @return 角色数据集合信息
     */
    @Override
    @DataScope(deptAlias = "d")
    public List<SysRole> selectRoleList(SysRole role) {
        return roleMapper.selectRoleList(role);
    }

    /**
     * 根据租户类型查询角色列表
     *
     * @param role 角色信息
     * @return
     */
    @TemporaryIgnoreTables
    @Override
    @DataScope(deptAlias = "d")
    public List<SysRole> selectRoleListByTenantType(SysRole role) {
        if (1 == SecurityUtils.getTenantType()) {
            SecurityUtils.setTemporaryIgnoreTables(Arrays.asList("d", "r", "ur", "u", "uu", "sur", "st", "sys_role",
                    "sys_user_role", "sys_user", "sys_dept"));
            SecurityUtils.setTenantId(role.getTenantId().toString());
        }
        role.setIsPage("0");
        role.setAdminRole(false);
        List<SysRole> sysRoleList = roleMapper.selectRoleList(role);
        sysRoleList.stream().filter(sysRole -> StrUtils.isNotEmpty(sysRole.getTenantName())).forEach(sysRole -> sysRole.setRoleName(sysRole.getTenantName() + "-" + sysRole.getRoleName()));
        return new ArrayList<>(sysRoleList.stream().sorted(Comparator.comparing(SysRole::getRoleSort)).collect(Collectors.toList()));
    }

    /**
     * 根据用户ID查询角色
     *
     * @param userId 用户ID
     * @return 角色列表
     */

    @Override
    public List<SysRole> selectRolesByUserId(Long userId) {
        List<SysRole> userRoles = roleMapper.selectRolePermissionByUserId(userId);
        return userRoles;
    }

    /**
     * 根据用户ID查询权限
     *
     * @param userId 用户ID
     * @return 权限列表
     */
    @Override
    public Set<String> selectRolePermissionByUserId(Long userId) {
        List<SysRole> perms = roleMapper.selectRolePermissionByUserId(userId);
        Set<String> permsSet = new HashSet<>();
        for (SysRole perm : perms) {
            if (StrUtils.isNotNull(perm)) {
                permsSet.addAll(Arrays.asList(perm.getRoleKey().trim().split(",")));
            }
        }
        return permsSet;
    }

    /**
     * 查询所有角色
     *
     * @return 角色列表
     */
    @Override
    public List<SysRole> selectRoleAll() {
        return SpringUtils.getAopProxy(this).selectRoleList(new SysRole());
    }

    /**
     * 根据用户ID获取角色选择框列表
     *
     * @param userId 用户ID
     * @return 选中角色ID列表
     */
    @Override
    public List<Long> selectRoleListByUserId(Long userId) {
        return roleMapper.selectRoleListByUserId(userId);
    }

    /**
     * 通过角色ID查询角色
     *
     * @param roleId 角色ID
     * @return 角色对象信息
     */
    @Override
    public SysRole selectRoleById(Long roleId) {
        return roleMapper.selectRoleById(roleId);
    }

    /**
     * 校验角色名称是否唯一
     *
     * @param role 角色信息
     * @return 结果
     */
    @Override
    public boolean checkRoleNameUnique(SysRole role) {
        Long roleId = StrUtils.isNull(role.getRoleId()) ? -1L : role.getRoleId();
        SysRole info = roleMapper.checkRoleNameUnique(role.getRoleName());
        if (StrUtils.isNotNull(info) && info.getRoleId().longValue() != roleId.longValue()) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验角色权限是否唯一
     *
     * @param role 角色信息
     * @return 结果
     */
    @Override
    public boolean checkRoleKeyUnique(SysRole role) {
        Long roleId = StrUtils.isNull(role.getRoleId()) ? -1L : role.getRoleId();
        SysRole info = roleMapper.checkRoleKeyUnique(role.getRoleKey());
        if (StrUtils.isNotNull(info) && info.getRoleId().longValue() != roleId.longValue()) {
            return UserConstants.NOT_UNIQUE;
        }
        return UserConstants.UNIQUE;
    }

    /**
     * 校验角色是否允许操作
     *
     * @param role 角色信息
     */
    @Override
    public void checkRoleAllowed(SysRole role) {
        if (StrUtils.isNotNull(role.getRoleId()) && role.isAdmin()) {
            throw new ServiceException("不允许操作超级管理员角色");
        }
    }

    /**
     * 校验角色是否有数据权限
     *
     * @param roleId 角色id
     */
    @Override
    public void checkRoleDataScope(Long roleId) {
        if (!SysUser.isAdmin(SecurityUtils.getUserId())) {
            SysRole role = new SysRole();
            role.setRoleId(roleId);
            List<SysRole> roles = SpringUtils.getAopProxy(this).selectRoleList(role);
            if (StrUtils.isEmpty(roles)) {
                throw new ServiceException("没有权限访问角色数据！");
            }
        }
    }

    /**
     * 通过角色ID查询角色使用数量
     *
     * @param roleId 角色ID
     * @return 结果
     */
    @Override
    public int countUserRoleByRoleId(Long roleId) {
        return userRoleMapper.countUserRoleByRoleId(roleId);
    }

    /**
     * 新增保存角色信息
     *
     * @param role 角色信息
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int insertRole(SysRole role) {
        // 新增角色信息
        role.setRoleKey(UuidUtils.generateUuid());
        roleMapper.insertRole(role);
        return insertRoleMenu(role);
    }

    /**
     * 修改保存角色信息
     *
     * @param role 角色信息
     * @return 结果
     */
    @Override
    public int updateRole(SysRole role) {
        // 修改角色信息
        roleMapper.updateRole(role);
        // 删除角色与菜单关联
        roleMenuMapper.deleteRoleMenuByRoleId(role.getRoleId());
        return insertRoleMenu(role);
    }

    /**
     * 通过角色Id获取对应的用户Id
     *
     * @return 结果
     */
    @Override
    public List<Long> selectUserIdListByRoleId(Long roleId) {
        List<SysUserRole> sysUserRoles = userRoleMapper.selectUserIdListByRoleId(roleId);
        return Objects.nonNull(sysUserRoles) && !sysUserRoles.isEmpty() ?
                sysUserRoles.stream().map(SysUserRole::getUserId).collect(Collectors.toList())
                : new ArrayList<>();
    }

    /**
     * 修改角色状态
     *
     * @param role 角色信息
     * @return 结果
     */
    @Override
    public int updateRoleStatus(SysRole role) {
        return roleMapper.updateRole(role);
    }

    /**
     * 修改数据权限信息
     *
     * @param role 角色信息
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int authDataScope(SysRole role) {
        // 修改角色信息
        roleMapper.updateRole(role);
        // 删除角色与部门关联
        roleDeptMapper.deleteRoleDeptByRoleId(role.getRoleId());
        // 新增角色和部门信息（数据权限）
        return insertRoleDept(role);
    }

    /**
     * 新增角色菜单信息
     *
     * @param role 角色对象
     */
    public int insertRoleMenu(SysRole role) {
        if (1 == SecurityUtils.getTenantType()) {
            SecurityUtils.setTenantId(String.valueOf(role.getTenantId()));
            List<String> removeTemporaryIgnoreTables = new ArrayList<>();
            removeTemporaryIgnoreTables.add("sys_role_menu");
            SecurityUtils.removeTemporaryIgnoreTables(removeTemporaryIgnoreTables);
        }
        int rows = 1;
        // 新增用户与角色管理
        List<SysRoleMenu> list = new ArrayList<SysRoleMenu>();
        for (Long menuId : role.getMenuIds()) {
            SysRoleMenu rm = new SysRoleMenu();
            rm.setRoleId(role.getRoleId());
            rm.setMenuId(menuId);
            list.add(rm);
        }
        if (!list.isEmpty()) {
            rows = roleMenuMapper.batchRoleMenu(list);
        }
        return rows;
    }

    /**
     * 新增角色部门信息(数据权限)
     *
     * @param role 角色对象
     */
    public int insertRoleDept(SysRole role) {
        int rows = 1;
        // 新增角色与部门（数据权限）管理
        List<SysRoleDept> list = new ArrayList<SysRoleDept>();
        for (Long deptId : role.getDeptIds()) {
            SysRoleDept rd = new SysRoleDept();
            rd.setRoleId(role.getRoleId());
            rd.setDeptId(deptId);
            list.add(rd);
        }
        if (list.size() > 0) {
            rows = roleDeptMapper.batchRoleDept(list);
        }
        return rows;
    }

    /**
     * 通过角色ID删除角色
     *
     * @param roleId 角色ID
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int deleteRoleById(Long roleId) {
        // 删除角色与菜单关联
        roleMenuMapper.deleteRoleMenuByRoleId(roleId);
        // 删除角色与部门关联
        roleDeptMapper.deleteRoleDeptByRoleId(roleId);
        return roleMapper.deleteRoleById(roleId);
    }

    /**
     * 批量删除角色信息
     *
     * @param roleIds 需要删除的角色ID
     * @return 结果
     */
    @TemporaryIgnoreTables
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int deleteRoleByIds(Long[] roleIds) {
        if (1 == SecurityUtils.getTenantType()) {
            SecurityUtils.setTemporaryIgnoreTables(Arrays.asList("d", "r", "ur", "u", "uu", "sur", "sys_role",
                    "sys_user_role", "sys_user", "sys_dept"));
        }

        for (Long roleId : roleIds) {
            checkRoleAllowed(new SysRole(roleId));
            checkRoleDataScope(roleId);
            SysRole role = selectRoleById(roleId);
            if (countUserRoleByRoleId(roleId) > 0) {
                throw new ServiceException(String.format("%1$s已分配,不能删除", role.getRoleName()));
            }
        }
        // 删除角色与菜单关联
        roleMenuMapper.deleteRoleMenu(roleIds);
        // 删除角色与部门关联
        roleDeptMapper.deleteRoleDept(roleIds);
        return roleMapper.deleteRoleByIds(roleIds);
    }

    /**
     * 取消授权用户角色
     *
     * @param userRole 用户和角色关联信息
     * @return 结果
     */
    @Override
    public int deleteAuthUser(SysUserRole userRole) {
        return userRoleMapper.deleteUserRoleInfo(userRole);
    }

    /**
     * 批量取消授权用户角色
     *
     * @param roleId  角色ID
     * @param userIds 需要取消授权的用户数据ID
     * @return 结果
     */
    @Override
    public int deleteAuthUsers(Long roleId, Long[] userIds) {
        return userRoleMapper.deleteUserRoleInfos(roleId, userIds);
    }

    @Override
    public int deleteAuthUsersByTenantId(Long roleId, Long[] userIds) {
        return userRoleMapper.deleteUserRoleInfosByTenantId(roleId, userIds);
    }

    /**
     * 批量选择授权用户角色
     *
     * @param roleId  角色ID
     * @param userIds 需要授权的用户数据ID
     * @return 结果
     */
    @Override
    public int insertAuthUsers(Long roleId, Long[] userIds) {
        // 新增用户与角色管理
        List<SysUserRole> list = new ArrayList<>();
        for (Long userId : userIds) {
            SysUserRole ur = new SysUserRole();
            ur.setUserId(userId);
            ur.setRoleId(roleId);
            list.add(ur);
        }
        return userRoleMapper.batchUserRole(list);
    }

    @Override
    public int insertAuthUsersByTenantId(Long roleId, Long[] userIds) {
        // 新增用户与角色管理
        List<SysUserRole> list = new ArrayList<SysUserRole>();
        for (Long userId : userIds) {
            SysUserRole ur = new SysUserRole();
            ur.setUserId(userId);
            ur.setRoleId(roleId);
            list.add(ur);
        }
        return userRoleMapper.batchUserRole(list);
    }
}
